const Partitioner = require('./partitioner.js')
const { container, event, recorder, config } = require('codeceptjs')

const testPartition = []
module.exports = function ({
  filterFn,
  suiteFilePath,
  project = process.env.CI_PROJECT_PATH_SLUG,
  reference = process.env.CI_COMMIT_REF_SLUG,
  suiteName = process.env.FILTER_SUITE_NAME || 'default',
  fallbackReference = process.env.CI_DEFAULT_BRANCH,
  nodeIndex = (Number(process.env.CI_NODE_INDEX) || 1) - 1,
  nodeTotal = Number(process.env.CI_NODE_TOTAL) || 1,
  suiteServiceUrl = process.env.SUITE_SERVICE_URL,
  pipelineId = process.env.CI_PIPELINE_ID,
  jwtSecret = process.env.SUITE_SERVICE_JWT_SECRET,
  nodePrefix = config.get('name').toLowerCase().replace(/[^a-z0-9]+/g, '-'),
  partitionPrefix = process.env.CI_JOB_STAGE ? `${process.env.CI_JOB_STAGE.replace(/\s+/g, '-')}-` : 'e2e-'
}) {
  filterFn = typeof filterFn === 'function' ? filterFn : testCase => testPartition.includes(testCase.fullTitle())
  if (container.mocha().options.invert) {
    const oldFilter = filterFn
    filterFn = t => !oldFilter(t)
  }

  const partitioner = new Partitioner({
    jwtSecret,
    suiteServiceUrl,
    suiteFilePath,
    project,
    suiteName,
    partitionId: pipelineId,
    reference,
    fallbackReference,
    nodePrefix,
    partitionPrefix
  })

  event.dispatcher.on(event.all.before, function () {
    recorder.startUnlessRunning()
    recorder.add('apply filter', async () => {
      const mocha = container.mocha()
      partitioner.grep = mocha.options?.grep
      partitioner.allTests = mocha.suite.suites.reduce((allTests, mochaSuite) =>
        allTests.concat(mochaSuite.tests), [])

      testPartition.push(...await partitioner.getPartition(nodeIndex, nodeTotal))
      mocha.suite.suites.forEach(suite => {
        suite.tests = suite.tests.filter(filterFn)
      })
    })
  })

  event.dispatcher.on(event.test.finished, function (test) {
    partitioner.updateState(test)
  })

  event.dispatcher.on(event.test.passed, function (test) {
    partitioner.updateDuration(test)
  })

  event.dispatcher.on(event.all.result, async function () {
    try {
      await partitioner.postDurations()
    } catch (error) {
      console.error(error)
    }
  })
}
