import PO from 'pofile'

import type { GettextPluginOptions } from '../index.js'
import type { POHeaders, PODictionary, POItem } from '../util/util.js'
import { itemKey } from '../util/util.js'

export type PORenderedChunk = PODictionary & { headers: POHeaders }

function isFuzzy (item: POItem): boolean {
  return !item.flags.fuzzy
}

// test if translations are blank/empty
function hasTranslation (item: POItem): boolean {
  return item.msgstr.join('').length > 0
}

export function render (po: PO, { includeFuzzy }: GettextPluginOptions): PORenderedChunk {
  const dict: PODictionary = {}
  for (const item of po.items) {
    if ((includeFuzzy || isFuzzy(item)) && !item.obsolete && hasTranslation(item)) {
      dict[itemKey(item)] = item.msgstr
    }
  }
  return Object.assign({ headers: po.headers }, dict)
}

export function namespacesFrom (items: POItem[]): string[] {
  const namespaces = new Set<string>()
  for (const item of items) {
    for (const reference of item.references) {
      const matches = reference.match(/module:([^\s]+)/)
      if (matches) {
        namespaces.add(matches[1])
      }
    }
  }
  return Array.from(namespaces)
}

export function parsePoFile (file: string): Promise<PO> {
  return new Promise((resolve, reject) => {
    PO.load(file, (err, po) => err ? reject(err) : resolve(po))
  })
}
